@@ -1,6 +1,13 @@
+from __future__ import annotations
+
+import sys
+
+assert sys.platform == "win32"
+
+from contextlib import contextmanager
 from ctypes import windll
 from ctypes.wintypes import HANDLE
-from typing import Callable, ContextManager, List
+from typing import Callable, ContextManager, Iterator
 
 from prompt_toolkit.eventloop.win32 import create_win32_event
 
@@ -31,7 +38,7 @@ class Win32PipeInput(_Win32InputBase, PipeInput):
 
     _id = 0
 
-    def __init__(self) -> None:
+    def __init__(self, _event: HANDLE) -> None:
         super().__init__()
         # Event (handle) for registering this input in the event loop.
         # This event is set when there is data available to read from the pipe.
@@ -43,13 +50,22 @@ class Win32PipeInput(_Win32InputBase, PipeInput):
         self._closed = False
 
         # Parser for incoming keys.
-        self._buffer: List[KeyPress] = []  # Buffer to collect the Key objects.
+        self._buffer: list[KeyPress] = []  # Buffer to collect the Key objects.
         self.vt100_parser = Vt100Parser(lambda key: self._buffer.append(key))
 
         # Identifier for every PipeInput for the hash.
         self.__class__._id += 1
         self._id = self.__class__._id
 
+    @classmethod
+    @contextmanager
+    def create(cls) -> Iterator[Win32PipeInput]:
+        event = create_win32_event()
+        try:
+            yield Win32PipeInput(_event=event)
+        finally:
+            windll.kernel32.CloseHandle(event)
+
     @property
     def closed(self) -> bool:
         return self._closed
@@ -65,7 +81,7 @@ class Win32PipeInput(_Win32InputBase, PipeInput):
         "The handle used for registering this pipe in the event loop."
         return self._event
 
-    def attach(self, input_ready_callback: Callable) -> ContextManager[None]:
+    def attach(self, input_ready_callback: Callable[[], None]) -> ContextManager[None]:
         """
         Return a context manager that makes this input active in the current
         event loop.
@@ -79,7 +95,7 @@ class Win32PipeInput(_Win32InputBase, PipeInput):
         """
         return detach_win32_input(self)
 
-    def read_keys(self) -> List[KeyPress]:
+    def read_keys(self) -> list[KeyPress]:
         "Read list of KeyPress."
 
         # Return result.
@@ -87,11 +103,13 @@ class Win32PipeInput(_Win32InputBase, PipeInput):
         self._buffer = []
 
         # Reset event.
-        windll.kernel32.ResetEvent(self._event)
+        if not self._closed:
+            # (If closed, the event should not reset.)
+            windll.kernel32.ResetEvent(self._event)
 
         return result
 
-    def flush_keys(self) -> List[KeyPress]:
+    def flush_keys(self) -> list[KeyPress]:
         """
         Flush pending keys and return them.
         (Used for flushing the 'escape' key.)
@@ -111,6 +129,9 @@ class Win32PipeInput(_Win32InputBase, PipeInput):
 
     def send_text(self, text: str) -> None:
         "Send text to the input."
+        if self._closed:
+            raise ValueError("Attempt to write into a closed pipe.")
+
         # Pass it through our vt100 parser.
         self.vt100_parser.feed(text)
 
@@ -124,12 +145,12 @@ class Win32PipeInput(_Win32InputBase, PipeInput):
         return DummyContext()
 
     def close(self) -> None:
-        "Close pipe handles."
-        windll.kernel32.CloseHandle(self._event)
+        "Close write-end of the pipe."
         self._closed = True
+        windll.kernel32.SetEvent(self._event)
 
     def typeahead_hash(self) -> str:
         """
         This needs to be unique for every `PipeInput`.
         """
-        return "pipe-input-%s" % (self._id,)
+        return f"pipe-input-{self._id}"
