@@ -436,6 +436,11 @@ class Vt100_Output(Output):
         # default, we don't change them.)
         self._cursor_shape_changed = False
 
+        # Don't hide/show the cursor when this was already done.
+        # (`None` means that we don't know whether the cursor is visible or
+        # not.)
+        self._cursor_visible: bool | None = None
+
     @classmethod
     def from_pty(
         cls,
@@ -651,10 +656,14 @@ class Vt100_Output(Output):
             self.write_raw("\x1b[%iD" % amount)
 
     def hide_cursor(self) -> None:
-        self.write_raw("\x1b[?25l")
+        if self._cursor_visible in (True, None):
+            self._cursor_visible = False
+            self.write_raw("\x1b[?25l")
 
     def show_cursor(self) -> None:
-        self.write_raw("\x1b[?12l\x1b[?25h")  # Stop blinking cursor and show.
+        if self._cursor_visible in (False, None):
+            self._cursor_visible = True
+            self.write_raw("\x1b[?12l\x1b[?25h")  # Stop blinking cursor and show.
 
     def set_cursor_shape(self, cursor_shape: CursorShape) -> None:
         if cursor_shape == CursorShape._NEVER_CHANGE:
